@@ -1,15 +1,13 @@
 import torch
-from torch.utils.model_zoo import load_url
-from enum import Enum
+from enum import IntEnum
 from skimage import io
-from skimage import color
 import numpy as np
+from distutils.version import LooseVersion
 
-from .models import FAN, ResNetDepth
 from .utils import *
 
 
-class LandmarksType(Enum):
+class LandmarksType(IntEnum):
     """Enum class defining the type of landmarks to detect.
 
     ``_2D`` - the detected points ``(x,y)`` are detected in a 2D space and follow the visible contour of the face
@@ -22,24 +20,30 @@ class LandmarksType(Enum):
     _3D = 3
 
 
-class NetworkSize(Enum):
+class NetworkSize(IntEnum):
     # TINY = 1
     # SMALL = 2
     # MEDIUM = 3
     LARGE = 4
 
-    def __new__(cls, value):
-        member = object.__new__(cls)
-        member._value_ = value
-        return member
 
-    def __int__(self):
-        return self.value
+default_model_urls = {
+    '2DFAN-4': 'https://www.adrianbulat.com/downloads/python-fan/2DFAN4-cd938726ad.zip',
+    '3DFAN-4': 'https://www.adrianbulat.com/downloads/python-fan/3DFAN4-4a694010b9.zip',
+    'depth': 'https://www.adrianbulat.com/downloads/python-fan/depth-6c4283c0e0.zip',
+}
 
 models_urls = {
-    '2DFAN-4': 'https://www.adrianbulat.com/downloads/python-fan/2DFAN4-11f355bf06.pth.tar',
-    '3DFAN-4': 'https://www.adrianbulat.com/downloads/python-fan/3DFAN4-7835d9f11d.pth.tar',
-    'depth': 'https://www.adrianbulat.com/downloads/python-fan/depth-2a464da4ea.pth.tar',
+    '1.6': {
+        '2DFAN-4': 'https://www.adrianbulat.com/downloads/python-fan/2DFAN4_1.6-c827573f02.zip',
+        '3DFAN-4': 'https://www.adrianbulat.com/downloads/python-fan/3DFAN4_1.6-ec5cf40a1d.zip',
+        'depth': 'https://www.adrianbulat.com/downloads/python-fan/depth_1.6-2aa3f18772.zip',
+    },
+    '1.5': {
+        '2DFAN-4': 'https://www.adrianbulat.com/downloads/python-fan/2DFAN4_1.5-a60332318a.zip',
+        '3DFAN-4': 'https://www.adrianbulat.com/downloads/python-fan/3DFAN4_1.5-176570af4d.zip',
+        'depth': 'https://www.adrianbulat.com/downloads/python-fan/depth_1.5-bc10f98e39.zip',
+    },
 }
 
 
@@ -51,7 +55,16 @@ class FaceAlignment:
         self.landmarks_type = landmarks_type
         self.verbose = verbose
 
+        if LooseVersion(torch.__version__) < LooseVersion('1.5.0'):
+            raise ImportError(f'Unsupported pytorch version detected. Minimum supported version of pytorch: 1.5.0\
+                            Either upgrade (recommended) your pytorch setup, or downgrade to face-alignment 1.2.0')
+
         network_size = int(network_size)
+        pytorch_version = torch.__version__
+        if 'dev' in pytorch_version:
+            pytorch_version = pytorch_version.rsplit('.', 2)[0]
+        else:
+            pytorch_version = pytorch_version.rsplit('.', 1)[0]
 
         if 'cuda' in device:
             torch.backends.cudnn.benchmark = True
@@ -62,27 +75,20 @@ class FaceAlignment:
         self.face_detector = face_detector_module.FaceDetector(device=device, verbose=verbose)
 
         # Initialise the face alignemnt networks
-        self.face_alignment_net = FAN(network_size)
         if landmarks_type == LandmarksType._2D:
             network_name = '2DFAN-' + str(network_size)
         else:
             network_name = '3DFAN-' + str(network_size)
-
-        fan_weights = load_url(models_urls[network_name], map_location=lambda storage, loc: storage)
-        self.face_alignment_net.load_state_dict(fan_weights)
+        self.face_alignment_net = torch.jit.load(
+            load_file_from_url(models_urls.get(pytorch_version, default_model_urls)[network_name]))
 
         self.face_alignment_net.to(device)
         self.face_alignment_net.eval()
 
         # Initialiase the depth prediciton network
         if landmarks_type == LandmarksType._3D:
-            self.depth_prediciton_net = ResNetDepth()
-
-            depth_weights = load_url(models_urls['depth'], map_location=lambda storage, loc: storage)
-            depth_dict = {
-                k.replace('module.', ''): v for k,
-                v in depth_weights['state_dict'].items()}
-            self.depth_prediciton_net.load_state_dict(depth_dict)
+            self.depth_prediciton_net = torch.jit.load(
+                load_file_from_url(models_urls.get(pytorch_version, default_model_urls)['depth']))
 
             self.depth_prediciton_net.to(device)
             self.depth_prediciton_net.eval()
@@ -113,24 +119,10 @@ class FaceAlignment:
             detected_faces {list of numpy.array} -- list of bounding boxes, one for each face found
             in the image (default: {None})
         """
-        if isinstance(image_or_path, str):
-            try:
-                image = io.imread(image_or_path)
-            except IOError:
-                print("error opening file :: ", image_or_path)
-                return None
-        elif isinstance(image_or_path, torch.Tensor):
-            image = image_or_path.detach().cpu().numpy()
-        else:
-            image = image_or_path
-
-        if image.ndim == 2:
-            image = color.gray2rgb(image)
-        elif image.ndim == 4:
-            image = image[..., :3]
+        image = get_image(image_or_path)
 
         if detected_faces is None:
-            detected_faces = self.face_detector.detect_from_image(image[..., ::-1].copy())
+            detected_faces = self.face_detector.detect_from_image(image.copy())
 
         if len(detected_faces) == 0:
             print("Warning: No faces were detected.")
@@ -138,7 +130,7 @@ class FaceAlignment:
 
         landmarks = []
         for i, d in enumerate(detected_faces):
-            center = torch.FloatTensor(
+            center = torch.tensor(
                 [d[2] - (d[2] - d[0]) / 2.0, d[3] - (d[3] - d[1]) / 2.0])
             center[1] = center[1] - (d[3] - d[1]) * 0.12
             scale = (d[2] - d[0] + d[3] - d[1]) / self.face_detector.reference_scale
@@ -150,13 +142,13 @@ class FaceAlignment:
             inp = inp.to(self.device)
             inp.div_(255.0).unsqueeze_(0)
 
-            out = self.face_alignment_net(inp)[-1].detach()
+            out = self.face_alignment_net(inp).detach()
             if self.flip_input:
-                out += flip(self.face_alignment_net(flip(inp))
-                            [-1].detach(), is_label=True)
-            out = out.cpu()
+                out += flip(self.face_alignment_net(flip(inp)).detach(), is_label=True)
+            out = out.cpu().numpy()
 
-            pts, pts_img = get_preds_fromhm(out, center, scale)
+            pts, pts_img = get_preds_fromhm(out, center.numpy(), scale)
+            pts, pts_img = torch.from_numpy(pts), torch.from_numpy(pts_img)
             pts, pts_img = pts.view(68, 2) * 4, pts_img.view(68, 2)
 
             if self.landmarks_type == LandmarksType._3D:
@@ -203,52 +195,15 @@ class FaceAlignment:
         landmarks = []
         # A batch for each frame
         for i, faces in enumerate(detected_faces):
-            landmark_set = []
-            for face in faces:
-                center = torch.FloatTensor(
-                    [(face[2] + face[0]) / 2.0,
-                     (face[3] + face[1]) / 2.0])
-
-                center[1] = center[1] - (face[3] - face[1]) * 0.12
-                scale = (face[2] - face[0] + face[3] - face[1]) / self.face_detector.reference_scale
-                image = image_batch[i].cpu().numpy()
-
-                image = image.transpose(1, 2, 0)
-
-                inp = crop(image, center, scale)
-                inp = torch.from_numpy(inp.transpose((2, 0, 1))).float()
-
-                inp = inp.to(self.device)
-                inp.div_(255.0).unsqueeze_(0)
-
-                out = self.face_alignment_net(inp)[-1].detach()
-                if self.flip_input:
-                    out += flip(self.face_alignment_net(flip(inp))
-                                [-1].detach(), is_label=True)  # patched inp_batch undefined variable error
-                out = out.cpu()
-                pts, pts_img = get_preds_fromhm(out, center, scale)
-
-                # Added 3D landmark support
-                if self.landmarks_type == LandmarksType._3D:
-                    pts, pts_img = pts.view(68, 2) * 4, pts_img.view(68, 2)
-                    heatmaps = np.zeros((68, 256, 256), dtype=np.float32)
-                    for i in range(68):
-                        if pts[i, 0] > 0:
-                            heatmaps[i] = draw_gaussian(
-                                heatmaps[i], pts[i], 2)
-                    heatmaps = torch.from_numpy(
-                        heatmaps).unsqueeze_(0)
-
-                    heatmaps = heatmaps.to(self.device)
-                    depth_pred = self.depth_prediciton_net(
-                        torch.cat((inp, heatmaps), 1)).data.cpu().view(68, 1)
-                    pts_img = torch.cat(
-                        (pts_img, depth_pred * (1.0 / (256.0 / (200.0 * scale)))), 1)
-                else:
-                    pts, pts_img = pts.view(-1, 68, 2) * 4, pts_img.view(-1, 68, 2)
-                landmark_set.append(pts_img.numpy())
-            if 0 != len(landmark_set):
+            landmark_set = self.get_landmarks_from_image(
+                image_batch[i].cpu().numpy().transpose(1, 2, 0),
+                detected_faces=faces
+            )
+            # Bacward compatibility
+            if landmark_set is not None:
                 landmark_set = np.concatenate(landmark_set, axis=0)
+            else:
+                landmark_set = []
             landmarks.append(landmark_set)
         return landmarks
 
